﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Kurs3
{
	/// <summary>
	/// Okno główne aplikacji
	/// </summary>
	public partial class MainForm : Form
	{
		private TDirectX directx;
		private int startTickCount = Environment.TickCount;

		public MainForm()
		{
			InitializeComponent();

			//Dwie poniższe linie konieczne tylko, gdy nie użyto panelu właściwości
			Load += new System.EventHandler(MainFormLoad);
			Paint += new System.Windows.Forms.PaintEventHandler(MainFormPaint);
			//
		}

		void MainFormLoad(object sender, EventArgs e)
		{
			SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.Opaque, true);
			directx = new TDirectX(this, true);
		}

		void MainFormPaint(object sender, PaintEventArgs e)
		{
			if (directx != null)
			{
				directx.Render(Environment.TickCount - startTickCount);
			}
			Invalidate();
		}
	}

	public class TDirectX
	{
		private Device device;
		private Control parent;
		private CustomVertex.TransformedColored[] vertex;

		/// <summary>
		/// Konstruktor obiektu obsługującego DirectX
		/// </summary>
		/// <param name="parent">Okno, które będzie celem renderowania grafiki</param>
		/// <param name="windowed">Czy renderować w oknie (true) czy pełnoekranowo (false)?</param>
		public TDirectX(Control parent, bool windowed)
		{
			this.parent = parent;

			PresentParameters presentParameters = new PresentParameters();
			presentParameters.Windowed = windowed;
			presentParameters.SwapEffect = SwapEffect.Discard;
			if (!windowed)
			{
				presentParameters.FullScreenRefreshRateInHz = 60;
				presentParameters.BackBufferWidth = 800;
				presentParameters.BackBufferHeight = 600;
				presentParameters.BackBufferCount = 1;
				presentParameters.BackBufferFormat = Format.X8R8G8B8;
				presentParameters.PresentFlag = PresentFlag.None;
			}

			device = new Device(0, DeviceType.Hardware, parent, CreateFlags.HardwareVertexProcessing, presentParameters);
			device.DeviceLost += new EventHandler(this.InvalidateDeviceObjects);
			device.DeviceReset += new EventHandler(this.RestoreDeviceObjects);
			device.Disposing += new EventHandler(this.DeleteDeviceObjects);
			device.DeviceResizing += new CancelEventHandler(this.EnvironmentResizing);
			Initialize();
		}

		/// <summary>
		/// Tworzenie wszystkich elementów sceny
		/// </summary>
		private void Initialize()
		{
			vertex = new CustomVertex.TransformedColored[4];
			
			vertex[0].Color = Color.Red.ToArgb();
			vertex[0].X=0;
			vertex[0].Y=0;
			vertex[0].Z=0;
			vertex[0].Rhw = 1;
			
			vertex[1].Color = Color.Green.ToArgb();
			vertex[1].X=64;
			vertex[1].Y=0;
			vertex[1].Z=0;
			vertex[1].Rhw = 1;
			
			vertex[2].Color = Color.Blue.ToArgb();
			vertex[2].Position = new Vector4(0, 64, 0, 1);
			
			vertex[3].Color = Color.Yellow.ToArgb();
			vertex[3].Position = new Vector4(64, 64, 0, 1);
		}

		/// <summary>
		/// Wykonanie renderowania
		/// </summary>
		/// <param name="time">Czas działania aplikacji</param>
		public void Render(float time)
		{
            device.RenderState.Lighting = false;
			device.Clear(ClearFlags.Target, Color.Blue, 1.0f, 0);
			device.BeginScene();
			
			device.VertexFormat = CustomVertex.TransformedColored.Format;
			device.DrawUserPrimitives(PrimitiveType.TriangleStrip, 2, vertex);
			
			device.EndScene();
			device.Present();
		}

		#region Obsługa zdarzeń
		/// <summary>
		/// Obsługa zdarzenia utraty dostępu do urządzenia
		/// </summary>
		protected virtual void InvalidateDeviceObjects(object sender, EventArgs e)
		{
		}

		/// <summary>
		/// Obsługa zdarzenia przywrócenia urządzenia
		/// </summary>
		protected virtual void RestoreDeviceObjects(object sender, EventArgs e)
		{
		}

		/// <summary>
		/// Obsługa zdarzenia usunięcia urządzenia
		/// </summary>
		protected virtual void DeleteDeviceObjects(object sender, EventArgs e)
		{
		}

		/// <summary>
		/// Obsługa zdarzenia zmiany wymiarów obrazu
		/// </summary>
		protected virtual void EnvironmentResizing(object sender, CancelEventArgs e)
		{
		}
		#endregion
	}
}
